import logging
import sys

import requests

from django_loki.formatters import LokiFormatter


class LokiHttpHandler(logging.Handler, object):
    """Python logging handler for Loki. Sends events over Http.
    :param host: The host of the loki server.
    :param port: The port of the loki server (default 3100).
    """

    def __init__(self, host: str = 'localhost', port: str = 3100, timeout: float = 0.5, protocol: str = 'http',
                 source: str = 'Loki', src_host: str = 'localhost', tz: str = 'UTC'):
        super(LokiHttpHandler, self).__init__()
        self._address = f'{protocol}://{host}:{port}'
        self._post_address = f'{self._address}/api/prom/push'
        self._tz = tz
        self._timeout = timeout
        self._source = source
        self._src_host = src_host

    def emit(self, record):
        """
        Emit a record.
        Send the record to the Web server as a percent-encoded dictionary
        """
        try:
            payload = self.formatter.format(record)
            res = requests.post(self._post_address, json=payload, timeout=self._timeout)
            if res.status_code != 204:
                sys.stderr.write('Loki occurs errors\n')
        except requests.exceptions.ReadTimeout:
            sys.stderr.write('Loki connect time out\n')
        except Exception:
            sys.stderr.write(f'{record.getMessage()}\n')

    def setFormatter(self, fmt: LokiFormatter) -> None:
        fmt.tz = self._tz
        fmt.source = self._source
        fmt.src_host = self._src_host
        self.formatter = fmt



import logging
import pytz
import socket

from datetime import datetime


class LokiFormatter(logging.Formatter, object):
    """
    Python logging formatter for Loki.
    """
    asctime_search = '%(asctime)'
    tz = 'UTC'
    source = 'Loki'
    src_host = 'localhost'

    def __init__(self, fmt: str, dfmt: str, style, fqdn=False):
        super(LokiFormatter, self).__init__()
        self.fmt = fmt
        self.dfmt = dfmt
        self.style = style
        if fqdn:
            self.host = socket.getfqdn()
        else:
            self.host = socket.gethostname()

    def format_timestamp(self, time):
        return datetime.fromtimestamp(time, tz=pytz.timezone(self.tz))

    def usesTime(self):
        return self.fmt.find(self.asctime_search) >= 0

    def formatMessage(self, record):
        try:
            return self.fmt % record.__dict__
        except KeyError as e:
            raise ValueError('Formatting field not found in record: %s' % e)

    def format(self, record):
        # Create message dict
        record.message = record.getMessage()
        if self.usesTime():
            record.asctime = self.formatTime(record, self.dfmt)
        s = self.formatMessage(record)
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            if s[-1:] != "\n":
                s = s + "\n"
            s = s + record.exc_text
        if record.stack_info:
            if s[-1:] != "\n":
                s = s + "\n"
            s = s + self.formatStack(record.stack_info)
        message = {
            'streams': [
                {
                    'labels': f'{{source="{self.source}",job="{record.name}",host="{self.src_host}"}}',
                    'entries': [
                        {
                            'ts': self.format_timestamp(record.created).isoformat('T'),
                            'line': s,
                        }
                    ]
                }
            ]
        }

        return message



from setuptools import setup
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-loki',
    version='0.1.4',
    packages=['django_loki'],
    url='https://github.com/zepc007/django-loki',
    license='MIT',
    author='zepc007',
    author_email='zepc007@gmail.com',
    description='logging handler with loki for django',
    long_description_content_type="text/markdown",
    long_description=long_description,
    keywords=['python', 'loki', 'grafana', 'logging', 'metrics'],
    install_requires=[
        'requests',
        'pytz',
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        "Environment :: Web Environment",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Development Status :: 3 - Alpha '
    ],
)
